import logging

bot = logging.getLogger("aiotele.bot")
types = logging.getLogger("aiotele.types")
event = logging.getLogger("aiotele.event")
middlewares = logging.getLogger("aiotele.middlewares")
webhook = logging.getLogger("aiotele.webhook")
scene = logging.getLogger("aiotele.scene")